

def _decodeVaruint32(stream):
    v = 0
    for i in range(5):
        b = stream.read(1)
        v += b[0] & 0x7f
        if b[0] & 0x80 == 0:
            break
        v = v << 7
    return v


def _encodeVaruint32(v: int):
    if (v < 0):
        raise ValueError("%d must be positive" % v)
    if (v > 2 ** 32 - 1):
        raise ValueError("%d is too large (max is 2^32-1)" % v)
    res = bytearray()
    while (v > 127):
        res += ((v & 0x7f) | 0x80).to_bytes(1, byteorder='big')
        v = v >> 7
    res += (v & 0x7f).to_bytes(1, byteorder='big')
    return res


def _encodeMessageToStream(stream, message):
    encodedMessageSize = _encodeVaruint32(message.ByteSize())
    stream.write(encodedMessageSize)
    stream.write(message.SerializeToString())



"""This module provides a Context to read sequences of tracking file.

Through the file.open() function one can get a context to read hermes
tracking data file sequences.

Example:
   .. code-block:: python

        import py_fort_hermes as fh

        with fh.file.open('tracking.0000.hermes') as f:
            try:
                for ro in f:
                    #do something with ro
            except fh.UnexpectedEndOfFileSequence as e:
                print("%s is corrupted"%e.segmentPath)

"""

import py_fort_hermes as fh
from py_fort_hermes import utils, check

import os
import gzip
import builtins


class Context:
    """
    A context manager class to open a sequence of hermes tracking file.

    It is an iterable to be used in a for loop.

    """
    __slots__ = ['width', 'height', 'path', 'followFile',
                 'filestream', '_line', 'allocateNewMessages']

    def __init__(self, filepath, followFile=True, allocateNewMessages=False):
        """Initializes a new Context

        Args:
            filepath (str): the first segment in the sequence to open
            followFile (bool): if False, will stop at the end of filepath,
                otherwise it will try to follow the file sequence
                until the last.
            allocateNewMessages (bool): if True, each call to `next` will
                allocate a new FrameReadout, which will lead to
                performance losses. Otherwise the same object will be
                used and returned.

        """

        self.followFile = followFile
        self.width = 0
        self.height = 0
        self.path = ''
        self._line = fh.Header_pb2.FileLine()
        self._openFile(filepath)
        self.allocateNewMessages = allocateNewMessages

    def __enter__(self):
        return self

    def close(self):
        """
        Closes the context manager manually
        """
        if self.filestream is not None:
            self.filestream.close()
        self.filestream = None

    def __iter__(self):
        return self

    def __next__(self):
        """
        Gets the next readout in the file sequence

        Returns:
            FrameReadout: the next frame readout

        Raises:
            StopIteration: when the last frame in the sequence was
                successfully raise
            UnexpectedEndOfFileSequence: if any error occurs before
                the last file in the sequence is successfully read

        """
        if self.filestream is None:
            raise StopIteration
        self._line.Clear()
        try:
            self._readMessage(self._line)
        except Exception as e:
            raise fh.UnexpectedEndOfFileSequence(segmentPath=self.path,
                                                 what="cannot decode line: %s" % e)
        if self._line.HasField('readout'):
            self._line.readout.width = self.width
            self._line.readout.height = self.height
            if self.allocateNewMessages is True:
                ro = fh.FrameReadout()
                ro.CopyFrom(self._line.readout)
                return ro
            else:
                return self._line.readout
        elif not self._line.HasField('footer'):
            raise fh.UnexpectedEndOfFileSequence(segmentPath=self.path,
                                                 what="got an empty line")

        if self._line.footer.next == '' or self.followFile == False:
            self.close()
            raise StopIteration

        newPath = os.path.join(os.path.dirname(self.path),
                               self._line.footer.next)
        self._openFile(newPath)
        return self.__next__()

    def __exit__(self, type, value, traceback):
        self.close()

    def _openFile(self, filepath):
        try:
            self.filestream = builtins.open(filepath + "unc", "rb")
        except Exception:
            self.filestream = gzip.open(filepath)

        h = fh.Header()
        try:
            self._readMessage(h)
        except Exception as e:
            raise fh.InternalError(fh.ErrorCode.FH_STREAM_NO_HEADER,
                                   "cannot parse header: %s" % e.message)

        check.CheckFileHeader(h)
        self.width = h.width
        self.height = h.height
        self.path = filepath

    def _readMessage(self, message):
        size = utils._decodeVaruint32(self.filestream)
        message.ParseFromString(self.filestream.read(size))


def open(filepath, followFile=True, allocateNewMessages=False):
    """Opens a sequence of hermes tracking file.

    Args:
        filepath : a path-like object to open
        followFile (bool): if True the sequence will be read until the
            last file, otherwise only filepath is read
        allocateNewMessages (bool): if True each call to next() will
            return a new FrameReadout. Otherwise, the same object will
            be Clear() and returned.
    Returns:
        Context: a context manager which is iterable
    """
    return Context(filepath, followFile, allocateNewMessages)



""" This modules provides a Context to read from a live leto instance

Example:
    .. code-block:: python

        import py_fort_hermes as fh

        with fh.network.connect("host") as c:
            for ro in c:
                # do something with c

"""
import py_fort_hermes as fh

import socket


class Context:
    """A context manager to open network connection to a live leto

    """

    __slots__ = [
        "_buffer",
        "_bytesRead",
        "_nextSize",
        "_s",
        "_blocking",
        "_ro",
    ]

    def __init__(self, host, port=4002, blocking=True, timeout=2.0, allocateNewMessages=False):
        """Initializes a connection to leto
        Args:
            host(str): a host to connect to
            port(int): the port to connect to
            blocking(bool): use a blocking connection
            timeout(float): timeout in second to use for waiting a message
            allocateNewMessages(bool): if True, each call to
                __next__() will return a newly allocated FrameReadout,
                with some performance onverhead. Otherwise the same
                object will be reused to avoid memory allocation.
        Raises:
            various error with socket.connect

        """
        self._buffer = bytearray()
        self._bytesRead = 0
        self._nextSize = 0
        self._s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._blocking = blocking
        if allocateNewMessages is True:
            self._ro = None
        else:
            self._ro = fh.FrameReadout()

        try:
            host = socket.gethostbyname(host)
            self._s.setblocking(blocking)
            if self._blocking:
                self._s.settimeout(timeout)

            self._s.connect((host, port))
            header = fh.Header()
            try:
                self._readMessage(header)
            except Exception as e:
                raise fh.InternalError(fh.ErrorCode.FH_STREAM_NO_HEADER,
                                       "could not read header: %s" % e)
            fh.check.CheckNetworkHeader(header)
        except:
            self._s.close()
            raise

    def __enter__(self):
        return self

    def close(self):
        if self._s is not None:
            self._s.close()
        self._s = None

    def __iter__(self):
        return self

    def __next__(self):
        """
        Gets the next FrameReadout in the stream

        Returns:
            FrameReadout: the next readout

        Raises:
            StopIteration: if no data was received after self.timeout
            socket.timeout: if some incomplete data have been received
                in self.timeout
            OSError: with EWOULDBLOCK or EAGAIN if non-blocking and no
                message where fully ready
        """
        ro = self._ro
        if ro is None:
            ro = fh.FrameReadout()
        self._readMessage(ro)
        return ro

    def __exit__(self, type, value, traceback):
        self.close()

    def _readMessage(self, message):
        if self._nextSize == 0:
            self._nextSize = self._readVaruint32()
            self._bytesRead = 0
        self._readAll(self._nextSize)
        message.ParseFromString(self._buffer[:self._bytesRead])
        self._nextSize = 0
        self._bytesRead = 0

    def _readReset(self):
        self._bytesRead = 0

    def _readAll(self, size):
        if self._bytesRead >= size:
            return

        if len(self._buffer) < size:
            self._buffer += bytearray(size - len(self._buffer))

        while self._bytesRead < size:
            b = self._s.recv_into(memoryview(self._buffer)[self._bytesRead:size],
                                  size - self._bytesRead)
            if b == 0 and self._blocking == True:
                raise StopIteration
            self._bytesRead += b

    def _readVaruint32(self):
        self._readAll(1)
        while self._buffer[self._bytesRead - 1] & 0x80 != 0:
            self._readAll(self._bytesRead + 1)

        v = self._buffer[0] & 0x7f
        for i in range(1, self._bytesRead):
            v = v << 7
            v += self._buffer[i] & 0x7f
        return v


def connect(host, port=4002, blocking=True, timeout=2.0, allocateNewMessages=False):
    """Connects to a leto host.

    Args:
        host(str): the host to connect to
        port(int): the port to connect to
        blocking(bool): use a blocking connection
        timeout(float): sets a timeout for IO
        allocateNewMessages(bool): if True, each call to
            next() will return a newly allocated FrameReadout,
            with some performance onverhead. Otherwise the same
            object will be reused to avoid memory allocation.
    Returns:
        Context: an iterable context manager for the connection
    Raises:
        Exception: If the connection could not be established
        InternalError: If the stream does not contain a valid header
    """
    return Context(host, port, blocking, timeout)



""" Various Errors returned by py_fort_hermes Contexts

"""

from enum import Enum


class ErrorCode(Enum):
    """Enumeration for internal error code

    """
    FH_NO_ERROR = 0
    """No errors occured"""
    FH_STREAM_NO_HEADER = 10001
    """The stream or file has no header"""
    FH_STREAM_WRONG_VERSION = 10002
    """The stream has an unsupported version"""
    FH_END_OF_STREAM = 10003
    """The stream reached the end"""
    FH_MESSAGE_DECODE_ERROR = 10004
    """The next message could not be decoded"""
    FH_COULD_NOT_CONNECT = 10005
    """could not connect to host"""
    FH_SOCKET_ERROR = 10006
    """network error"""
    FH_UNEXPECTED_END_OF_FILE_SEQUENCE = 10007
    """an error occured before the end of a file sequence"""


class InternalError(Exception):
    """Internal error raised by py_fort_hermes.

    Attributes:
        code (int): the ErrorCode for the internal error

    """

    def __init__(self, code: ErrorCode, message):
        self.message = message
        self.code = code
        super().__init__(self.message)


class UnexpectedEndOfFileSequence(Exception):
    """The reading of a sequence of file was interrupted by an error

    Attributes:
        segmentPath (str): the segment where the error occured

    """

    def __init__(self, what, segmentPath):
        self.segmentPath = segmentPath
        self.message = "Unexpected end of file sequence in '%s': %s" % (
            segmentPath, what)
        super().__init__(self.message)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Tag.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='Tag.proto',
  package='fort.hermes',
  syntax='proto3',
  serialized_options=b'H\003Z$github.com/formicidae-tracker/hermes',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\tTag.proto\x12\x0b\x66ort.hermes\"6\n\x03Tag\x12\n\n\x02ID\x18\x01 \x01(\r\x12\t\n\x01x\x18\x02 \x01(\x01\x12\t\n\x01y\x18\x03 \x01(\x01\x12\r\n\x05theta\x18\x04 \x01(\x01\x42(H\x03Z$github.com/formicidae-tracker/hermesb\x06proto3'
)




_TAG = _descriptor.Descriptor(
  name='Tag',
  full_name='fort.hermes.Tag',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ID', full_name='fort.hermes.Tag.ID', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='x', full_name='fort.hermes.Tag.x', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y', full_name='fort.hermes.Tag.y', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='theta', full_name='fort.hermes.Tag.theta', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26,
  serialized_end=80,
)

DESCRIPTOR.message_types_by_name['Tag'] = _TAG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Tag = _reflection.GeneratedProtocolMessageType('Tag', (_message.Message,), {
  'DESCRIPTOR' : _TAG,
  '__module__' : 'Tag_pb2'
  # @@protoc_insertion_point(class_scope:fort.hermes.Tag)
  })
_sym_db.RegisterMessage(Tag)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: FrameReadout.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from . import Tag_pb2 as Tag__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='FrameReadout.proto',
  package='fort.hermes',
  syntax='proto3',
  serialized_options=b'H\003Z$github.com/formicidae-tracker/hermes',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x12\x46rameReadout.proto\x12\x0b\x66ort.hermes\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x19google/protobuf/any.proto\x1a\tTag.proto\"\xcf\x03\n\x0c\x46rameReadout\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\x12\x0f\n\x07\x66rameID\x18\x02 \x01(\x03\x12\x1e\n\x04tags\x18\x03 \x03(\x0b\x32\x10.fort.hermes.Tag\x12(\n\x04time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\x05\x65rror\x18\x05 \x01(\x0e\x32\x1f.fort.hermes.FrameReadout.Error\x12\x15\n\rproducer_uuid\x18\x06 \x01(\t\x12\r\n\x05quads\x18\x07 \x01(\x05\x12\r\n\x05width\x18\x08 \x01(\x05\x12\x0e\n\x06height\x18\t \x01(\x05\x12,\n\x08\x63uestart\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12*\n\x06\x63ueend\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12(\n\nextensions\x18\x0c \x03(\x0b\x32\x14.google.protobuf.Any\"X\n\x05\x45rror\x12\x0c\n\x08NO_ERROR\x10\x00\x12\x14\n\x10PROCESS_OVERFLOW\x10\x01\x12\x13\n\x0fPROCESS_TIMEOUT\x10\x02\x12\x16\n\x12ILLUMINATION_ERROR\x10\x03\x42(H\x03Z$github.com/formicidae-tracker/hermesb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_any__pb2.DESCRIPTOR,Tag__pb2.DESCRIPTOR,])



_FRAMEREADOUT_ERROR = _descriptor.EnumDescriptor(
  name='Error',
  full_name='fort.hermes.FrameReadout.Error',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NO_ERROR', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PROCESS_OVERFLOW', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PROCESS_TIMEOUT', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ILLUMINATION_ERROR', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=482,
  serialized_end=570,
)
_sym_db.RegisterEnumDescriptor(_FRAMEREADOUT_ERROR)


_FRAMEREADOUT = _descriptor.Descriptor(
  name='FrameReadout',
  full_name='fort.hermes.FrameReadout',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='fort.hermes.FrameReadout.timestamp', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='frameID', full_name='fort.hermes.FrameReadout.frameID', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tags', full_name='fort.hermes.FrameReadout.tags', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time', full_name='fort.hermes.FrameReadout.time', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error', full_name='fort.hermes.FrameReadout.error', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='producer_uuid', full_name='fort.hermes.FrameReadout.producer_uuid', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='quads', full_name='fort.hermes.FrameReadout.quads', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='width', full_name='fort.hermes.FrameReadout.width', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='height', full_name='fort.hermes.FrameReadout.height', index=8,
      number=9, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cuestart', full_name='fort.hermes.FrameReadout.cuestart', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cueend', full_name='fort.hermes.FrameReadout.cueend', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='extensions', full_name='fort.hermes.FrameReadout.extensions', index=11,
      number=12, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _FRAMEREADOUT_ERROR,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=107,
  serialized_end=570,
)

_FRAMEREADOUT.fields_by_name['tags'].message_type = Tag__pb2._TAG
_FRAMEREADOUT.fields_by_name['time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_FRAMEREADOUT.fields_by_name['error'].enum_type = _FRAMEREADOUT_ERROR
_FRAMEREADOUT.fields_by_name['cuestart'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_FRAMEREADOUT.fields_by_name['cueend'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_FRAMEREADOUT.fields_by_name['extensions'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_FRAMEREADOUT_ERROR.containing_type = _FRAMEREADOUT
DESCRIPTOR.message_types_by_name['FrameReadout'] = _FRAMEREADOUT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FrameReadout = _reflection.GeneratedProtocolMessageType('FrameReadout', (_message.Message,), {
  'DESCRIPTOR' : _FRAMEREADOUT,
  '__module__' : 'FrameReadout_pb2'
  # @@protoc_insertion_point(class_scope:fort.hermes.FrameReadout)
  })
_sym_db.RegisterMessage(FrameReadout)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import py_fort_hermes as fh


def CheckVersion(version):
    if version not in {(0, 1), (0, 2), (0, 3), (0, 4), (0, 5)}:
        raise fh.InternalError(fh.ErrorCode.FH_STREAM_WRONG_VERSION,
                               "unsupported version %d.%d" % (version[0], version[1]))


def CheckFileHeader(header):
    CheckVersion((header.version.vmajor, header.version.vminor))
    if header.type != fh.Header.File:
        raise fh.InternalError(fh.ErrorCode.FH_STREAM_NO_HEADER,
                               "wrong header type")
    if header.width == 0:
        raise fh.InternalError(fh.ErrorCode.FH_STREAM_NO_HEADER,
                               "missing width")
    if header.height == 0:
        raise fh.InternalError(fh.ErrorCode.FH_STREAM_NO_HEADER,
                               "missing height")


def CheckNetworkHeader(header):
    CheckVersion((header.version.vmajor, header.version.vminor))
    if header.type != fh.Header.Network:
        raise fh.InternalError(fh.ErrorCode.FH_STREAM_NO_HEADER,
                               "wrong header type")



import setuptools
import setuptools.command.build_py
import os
import subprocess
import sys

from distutils.spawn import find_executable


with open("./README.rst", "r", encoding="utf-8") as fh:
    long_description = fh.read()


class BuildProtoCommand(setuptools.command.build_py.build_py):
    """A Command that generate required protobuf messages"""

    if 'PROTOC' in os.environ and os.path.exists(os.environ['PROTOC']):
        protoc = os.environ['PROTOC']
    else:
        protoc = find_executable("protoc")
        sed = find_executable("sed")

    def generate_proto(source):
        """Invokes the Protocol Compiler to generate a _pb2.py from the given
        .proto file.  Does nothing if the output already exists and is newer than
        the input."""

        output = os.path.join('py_fort_hermes', os.path.basename(
            source).replace(".proto", "_pb2.py"))

        if (not os.path.exists(output) or
            (os.path.exists(source) and
             os.path.getmtime(source) > os.path.getmtime(output))):
            print("Generating %s..." % output)

        if not os.path.exists(source):
            sys.stderr.write("Can't find required file: %s\n" % source)
            sys.exit(-1)

        if BuildProtoCommand.protoc is None:
            sys.stderr.write(
                "protoc is not found. Please install the binary package.\n")
            sys.exit(-1)

        protoc_command = [BuildProtoCommand.protoc,
                          "-Ipy_fort_hermes/protobuf", "--python_out=py_fort_hermes", source]
        if subprocess.call(protoc_command) != 0:
            sys.exit(-1)

        sed_command = [BuildProtoCommand.sed,
                       's/^import \(.*_pb2\) as \(.*_pb2\)$/from . import \\1 as \\2/g', "-i", output]
        if subprocess.call(sed_command) != 0:
            sys.exit(-1)

    def run(self):
        BuildProtoCommand.generate_proto('py_fort_hermes/protobuf/Tag.proto')
        BuildProtoCommand.generate_proto(
            'py_fort_hermes/protobuf/FrameReadout.proto')
        BuildProtoCommand.generate_proto(
            'py_fort_hermes/protobuf/Header.proto')
        setuptools.command.build_py.build_py.run(self)


setuptools.setup(
    name="formicidae_tracker_hermes",
    author="Alexandre Tuleu",
    version_config={
        "dev_template": "{tag}.post{ccount}",
    },
    author_email="alexandre.tuleu.2005@polytechnique.org",
    description="FORmicidae Tracker hermes python implementation",
    long_description=long_description,
    url="https://github.com/formicidae-tracker/hermes",
    project_urls={
        "Bug Tracker": "https://github.com/formicidae-tracker/hermes/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: OS Independent",
    ],
    packages=["py_fort_hermes"],
    package_dir={"": "."},
    package_data={"py_fort_hermes": ["protobuf/*.proto"]},
    python_requires=">=3.6",
    cmdclass={
        'build_py': BuildProtoCommand,
    },
    setup_requires=['setuptools-git-versioning'],
    install_requires=[
        "protobuf",
    ],
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Header.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import FrameReadout_pb2 as FrameReadout__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='Header.proto',
  package='fort.hermes',
  syntax='proto3',
  serialized_options=b'H\003Z$github.com/formicidae-tracker/hermes',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0cHeader.proto\x12\x0b\x66ort.hermes\x1a\x12\x46rameReadout.proto\")\n\x07Version\x12\x0e\n\x06vmajor\x18\x01 \x01(\r\x12\x0e\n\x06vminor\x18\x02 \x01(\r\"\xbf\x01\n\x06Header\x12%\n\x07version\x18\x01 \x01(\x0b\x32\x14.fort.hermes.Version\x12&\n\x04type\x18\x02 \x01(\x0e\x32\x18.fort.hermes.Header.Type\x12\x10\n\x08previous\x18\x03 \x01(\t\x12\r\n\x05width\x18\x04 \x01(\x05\x12\x0e\n\x06height\x18\x05 \x01(\x05\x12\x16\n\x0e\x65xtensionTypes\x18\x06 \x03(\t\"\x1d\n\x04Type\x12\x0b\n\x07Network\x10\x00\x12\x08\n\x04\x46ile\x10\x01\"\x16\n\x06\x46ooter\x12\x0c\n\x04next\x18\x01 \x01(\t\"[\n\x08\x46ileLine\x12*\n\x07readout\x18\x01 \x01(\x0b\x32\x19.fort.hermes.FrameReadout\x12#\n\x06\x66ooter\x18\x02 \x01(\x0b\x32\x13.fort.hermes.FooterB(H\x03Z$github.com/formicidae-tracker/hermesb\x06proto3'
  ,
  dependencies=[FrameReadout__pb2.DESCRIPTOR,])



_HEADER_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='fort.hermes.Header.Type',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='Network', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='File', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=255,
  serialized_end=284,
)
_sym_db.RegisterEnumDescriptor(_HEADER_TYPE)


_VERSION = _descriptor.Descriptor(
  name='Version',
  full_name='fort.hermes.Version',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='vmajor', full_name='fort.hermes.Version.vmajor', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='vminor', full_name='fort.hermes.Version.vminor', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=49,
  serialized_end=90,
)


_HEADER = _descriptor.Descriptor(
  name='Header',
  full_name='fort.hermes.Header',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='fort.hermes.Header.version', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='fort.hermes.Header.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='previous', full_name='fort.hermes.Header.previous', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='width', full_name='fort.hermes.Header.width', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='height', full_name='fort.hermes.Header.height', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='extensionTypes', full_name='fort.hermes.Header.extensionTypes', index=5,
      number=6, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _HEADER_TYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=93,
  serialized_end=284,
)


_FOOTER = _descriptor.Descriptor(
  name='Footer',
  full_name='fort.hermes.Footer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='next', full_name='fort.hermes.Footer.next', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=286,
  serialized_end=308,
)


_FILELINE = _descriptor.Descriptor(
  name='FileLine',
  full_name='fort.hermes.FileLine',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='readout', full_name='fort.hermes.FileLine.readout', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='footer', full_name='fort.hermes.FileLine.footer', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=310,
  serialized_end=401,
)

_HEADER.fields_by_name['version'].message_type = _VERSION
_HEADER.fields_by_name['type'].enum_type = _HEADER_TYPE
_HEADER_TYPE.containing_type = _HEADER
_FILELINE.fields_by_name['readout'].message_type = FrameReadout__pb2._FRAMEREADOUT
_FILELINE.fields_by_name['footer'].message_type = _FOOTER
DESCRIPTOR.message_types_by_name['Version'] = _VERSION
DESCRIPTOR.message_types_by_name['Header'] = _HEADER
DESCRIPTOR.message_types_by_name['Footer'] = _FOOTER
DESCRIPTOR.message_types_by_name['FileLine'] = _FILELINE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Version = _reflection.GeneratedProtocolMessageType('Version', (_message.Message,), {
  'DESCRIPTOR' : _VERSION,
  '__module__' : 'Header_pb2'
  # @@protoc_insertion_point(class_scope:fort.hermes.Version)
  })
_sym_db.RegisterMessage(Version)

Header = _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
  'DESCRIPTOR' : _HEADER,
  '__module__' : 'Header_pb2'
  # @@protoc_insertion_point(class_scope:fort.hermes.Header)
  })
_sym_db.RegisterMessage(Header)

Footer = _reflection.GeneratedProtocolMessageType('Footer', (_message.Message,), {
  'DESCRIPTOR' : _FOOTER,
  '__module__' : 'Header_pb2'
  # @@protoc_insertion_point(class_scope:fort.hermes.Footer)
  })
_sym_db.RegisterMessage(Footer)

FileLine = _reflection.GeneratedProtocolMessageType('FileLine', (_message.Message,), {
  'DESCRIPTOR' : _FILELINE,
  '__module__' : 'Header_pb2'
  # @@protoc_insertion_point(class_scope:fort.hermes.FileLine)
  })
_sym_db.RegisterMessage(FileLine)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



from . import file
from . import network

from py_fort_hermes.errors import ErrorCode, InternalError, UnexpectedEndOfFileSequence
from py_fort_hermes.Header_pb2 import Header, FileLine, Footer
from py_fort_hermes.Tag_pb2 import Tag
from py_fort_hermes.FrameReadout_pb2 import FrameReadout
